/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "IDLDefinition_Private.h"
#import "IDLFunction_Private.h"
#import "ObjectCell.h"
#import "IDLArgument.h"
#import "IDLInterface_Private.h"
#import "IDLViewer.h"

@implementation IDLFunction(Viewer)

- (NSImage*) cellImage
{
	return [NSImage imageNamed:@"function13"];
}

- (void) loadCell:cell
{
    [cell setLoaded:YES];
    [cell setLeaf:NO];
	[cell setImage:[self cellImage]];
    [cell setRepresentedObject:self];
}

- (void) fillMatrix:matrix
{
    int n = 0;
    NSEnumerator* enumerator = [_args objectEnumerator];
    IDLArgument* item;
    id cell;

    while(item = [enumerator nextObject]){
        [matrix addRow];
        [[item definition] loadCell:cell = [matrix cellAtRow:n++ column:0]];
        [cell setStringValue:[item name]];
    }
    if(_returnType){
        [matrix addRow];
        [_returnType loadCell:cell = [matrix cellAtRow:n++ column:0]];
        [cell setStringValue:@"$return"];
    }
    if([_contextList count] > 0){
        [matrix addRow];
        [_contextList loadCell:cell = [matrix cellAtRow:n++ column:0]];
        [cell setStringValue:@"$context"];
    }
    if([_exceptionList count] > 0){
        [matrix addRow];
        [_exceptionList loadCell:cell = [matrix cellAtRow:n++ column:0]];
        [cell setStringValue:@"$raises"];
    }
}

- (NSString*) category
{
    return @"- Operations -";
}

- (NSString*) title
{
    return @"Arguments";
}

- (NSString*) stringValue
{
	NSEnumerator* en = [_args objectEnumerator];
	IDLArgument* def;
	NSMutableArray* array = [NSMutableArray array];
	NSString* rt = _returnType?[[_returnType name]stringByAppendingString:@" "]:@"";
        NSString* factory = _isFactory?@"factory ":@"";
	while(def = [en nextObject])
		[array addObject:[NSString stringWithFormat:@"%@ %@", 
                    [def stringMode], [def name]]];

	return [NSString stringWithFormat:@"%@%@%@(%@)",
		factory,rt,_name,[array componentsJoinedByString:@", "]];
}


- (BOOL) writeDefinitionToPboard:(NSPasteboard*)pboard
{
    return NO;
}

@end
