/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "StockController.h"
#import <AppKit/AppKit.h>
#import <ADORB/ADORB.h>
#import "StockWatch.h"
#import "StockAdmin.h"

@implementation StockController

- init
{
	self = [super init];
	_watchArray = [[NSMutableArray alloc]init];
	return self;
}

- (void) dealloc
{
	[_watchArray release];
	[_admin release];
	[_server release];
	[super dealloc];
}

- server
{
	return _server;
}

- (void)connect:(id)sender
{
	id nameService = [ORB resolveInitialReferences:@"NameService"];
	if(nameService){
		NSString *path = @"StockServer";
		[self willChangeValueForKey:@"server"];
		_server = [[nameService resolve:[IDLNameComponent nameComponentsForPath:path]]retain];
		[self didChangeValueForKey:@"server"];
		if(_server){
			[_server setDelegate:self];
			return;
		}
	}

    NSOpenPanel* panel = [NSOpenPanel openPanel];
    [panel setAllowsMultipleSelection:NO];
    [panel setTitle:@"Open IOR"];
    if([panel runModalForTypes:[NSArray arrayWithObject:@"ior"]]){
        NSArray* files = [panel filenames];
        NSString* ior = [[[NSString alloc]initWithContentsOfFile:[files objectAtIndex:0]]autorelease];
		[self willChangeValueForKey:@"server"];
        _server = [[CORBAObject objectWithIOR:ior]retain];
		[self didChangeValueForKey:@"server"];
		// set self the delegate to be notified when _server is invalidated
		[_server setDelegate:self];
    }
}

- (void)newWatch:(id)sender
{
    id watcher = [[StockWatch alloc]initWithController:self];
	[_watchArray addObject:watcher];
	[watcher release];
    [[watcher window]makeKeyAndOrderFront:self];
}

- (void)removeWatch:watch
{
	if(watch)
		[_watchArray removeObject:watch];
}

- (void)openStockAdmin:(id)sender
{
    if(!_admin)
        _admin = [[StockAdmin alloc]initWithController:self];
    [[_admin window]makeKeyAndOrderFront:self];
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    NSString* path = [[NSBundle mainBundle]pathForResource:@"Quote" ofType:@"idl"];
    NSArray* files = [NSArray arrayWithObject:path];
    [[ORB defaultORB]loadFiles:files];
    
    // Before any connection is established
    if([[NSUserDefaults standardUserDefaults]boolForKey:@"useMainThread"])
        [ORB setHandlesRequestsOnMainThread:YES];
    
    [self connect:self];
	[self newWatch:self];
}

- (BOOL)validateMenuItem:(NSMenuItem*)menuItem
{
    SEL action = [menuItem action];
    if(action == @selector(connect:))
        return _server == nil;
    else
        return _server != nil;
}

// CORBAObject sends this message to its delegate when invalidated
- (void) objectDidInvalidate:(NSNotification*)notification
{
	if([notification object] == _server){
		[self willChangeValueForKey:@"server"];
		[_server release];
		_server = nil;
		[self didChangeValueForKey:@"server"];
		NSRunAlertPanel(@"Connection is lost", @"Connection to the server is terminated", @"OK", nil, nil);
	}
}

@end
