/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLValueType_Private.h"
#import "IDLDefinition_Private.h"
#import "IDLValueClass.h"
#import "CORBACoding.h"
#import "IDLInterface.h"
#import "IDLObject.h"

@implementation IDLValueType

NSString* IDLValuetypeNeededForObject = @"IDLValuetypeNeededForObject";
static NSMutableDictionary* _cache = nil;
static id _defaultObjectClass = nil;
static NSString* _chunkDefaultsKey = @"IDLValueEncodeChunks";

+ (void) initialize
{
    if([IDLValueType class] == self){
		[[NSUserDefaults standardUserDefaults]registerDefaults:[NSDictionary dictionaryWithObjectsAndKeys:
			[NSNumber numberWithBool:YES], _chunkDefaultsKey, nil]];
        NSString* debug = [[NSUserDefaults standardUserDefaults]stringForKey:@"debug"];
        if(debug){
            NSRange r = [debug rangeOfString:@"V"];
            if(r.length > 0)
                [self setLogEnabled:YES];
        }
		[self setEncodeValuesUsingChunks:[[NSUserDefaults standardUserDefaults]boolForKey:_chunkDefaultsKey]];
        _defaultObjectClass = [IDLObject class];
    }
}

+ defaultObjectClass
{
    return _defaultObjectClass;
}

+ (void) setDefaultObjectClass:class
{
    if(class)
        _defaultObjectClass = class;
}

- initWithName:(NSString*)name
{
    self = [super init];
    _name = [name retain];
    [[NSNotificationCenter defaultCenter]addObserver:self 
            selector:@selector(setValuetypeFor:)
            name:IDLValuetypeNeededForObject
            object:nil];
    return self;
}

- (void) dealloc
{
    [[NSNotificationCenter defaultCenter]removeObserver:self];
    [_className release];
    [_support release];
    [_name release];
    [super dealloc];
}

+ (IDLValueType*)valueTypeWithName:(NSString*)name
{
    return [[[self alloc]initWithName:name]autorelease];
}

+ (IDLValueType*)valueType
{
    return [self valueTypeWithName:nil];
}

- (IDLValueClass*) classSupport
{
    if(_support == nil){
        _support = [[IDLValueClass alloc]initWithValueType:self];
        [_support setModifier:_modifier];
        return _support;
    }else{
        if([_support isKindOfClass:[IDLValueClass class]])
            return _support;
        else{
            NSLog(@"Trying to get class support for a box valuetype\n");
            return nil;
        }
        return nil;
    }
}

- (IDLValueClass*) _classSupport
{
    return _support;
}

- (BOOL) addMembers:(NSArray*)nameArray ofType:(IDLDefinition*)type withMode:(int)accessMode
{
    return [[self classSupport]addMembers:nameArray ofType:type withMode:accessMode];
}

- (void) setTypeDefinition:(IDLDefinition*)type
{
    if(_support == nil){
        _support = [type retain];
        if(_modifier != 0){
            NSLog(@"ValueBox cannot have modifier abstract or custom, ignored\n");
            _modifier = 0;
        }
    }else
        NSLog(@"Trying to redefine valuetype's type\n");
}

- (void) setModifier:(int)modifier
{
    _modifier = modifier;
}

- (void) setInheritance:(NSArray*)list truncatable:(BOOL)flag
{
    [[self classSupport]setInheritance:list truncatable:flag];
}

- (void) setSupportedInterfaces:(NSArray*)list
{
    [[self classSupport]setSupportedInterfaces:list];
}

- (void) addDefinition:(IDLDefinition*)def withName:(NSString*)name
{
    [[self classSupport]addDefinition:def withName:name];
}

- addDefinition:(IDLDefinition*)def withIdentifierArray:(NSArray*)identArray
{
    return [[self classSupport]addDefinition:def withIdentifierArray:identArray];
}

- (IDLDefinition*)_definitionNamed:(NSString*)name
{
    return [[self classSupport]_definitionNamed:name];
}
- (id <IDLDefinition>) localDefinitionNamed:(NSString*)name
{
    return [self _definitionNamed:name];
}
- (id <IDLDefinition>)definitionNamed:(NSString*)name
{
    if([_support isKindOfClass:[IDLValueClass class]])
        return (IDLDefinition*)[_support definitionNamed:name];
    else
        return nil;
}

- (id <IDLDefinitionScope>) parentScope
{
    if([_support respondsToSelector:_cmd])
        return [_support parentScope];
    else
        return nil;
}

- (IDLDefinitionScope*) rootScope
{
    if([_support respondsToSelector:_cmd])
        return [_support rootScope];
    else
        return nil;
}

- (void) setParentScope:(id<IDLDefinitionScope>)scope
{
    [[self classSupport]setParentScope:scope];
}

- (NSNumber*) enumValueForName:(NSString*)name andKey:(NSString*)key
{
    [self doesNotRecognizeSelector:_cmd];
    return nil;
}

- (id <IDLDefinition>) definitionForNamePath:(NSString*)path
{
    if([_support respondsToSelector:_cmd])
        return [_support definitionForNamePath:path];
    else
        return nil;
}

- (id <IDLDefinition>) definitionForTypeId:(NSString*)typeId
{
    if([_support respondsToSelector:_cmd])
        return [_support definitionForTypeId:typeId];
    else
        return nil;
}

- (NSString*)name
{
    return _name;
}

- (void) setName:(NSString*)name
{
    [_name release];
    _name = [name retain];
}

- (BOOL) isValueBox
{
    return (_support == nil || [_support isKindOfClass:[IDLValueClass class]])?NO:YES;
}

- (long) kind
{
    if([self isValueBox])
        return tk_value_box;
    else
        return tk_value;
}

- (void) setObjectClassName:(NSString*)string
{
    if(string != _className){
        [_className release];
        _className = [string copy];
    }
}
- (NSString*) objectClassName
{
    return _className;
}

- (BOOL) isAbstract
{
    return _modifier == vt_modifier_abstract;
}

- superValue
{
    if([_support respondsToSelector:_cmd])
        return [_support superValue];
    else
        return nil;
}

static id _valueBase = nil;

+ valueBase
{
    if(!_valueBase)
    	_valueBase = [[IDLDefinitionScope rootScope]definitionForNamePath:@"CORBA/ValueBase"];
    return _valueBase;
}

- (BOOL) isDerivedFrom:(IDLValueType*)value
{
    if(value == self || value == [IDLValueType valueBase])
        return YES;
    else{
        if([_support respondsToSelector:_cmd])
            return [_support isDerivedFrom:value];
        else
            return NO;
    }
}

// Check only abstract interfaces
// Value is not derived from a concrete interface
- (BOOL) isDerivedFromInterface:(IDLInterface*)interface
{
    if([interface isAbstract] && [_support respondsToSelector:_cmd])
        return [_support isDerivedFromInterface:interface];
    else
        return NO;
}

- (BOOL) _isDerivedFromIDLType:(IDLDefinition*)type
{
	if([type isKindOfClass:[IDLValueType class]])
		return [self isDerivedFrom:(IDLValueType*)type];
	else
	if([type isKindOfClass:[IDLInterface class]])
		return [self isDerivedFromInterface:(IDLInterface*)type];
	else
		return NO;
}

- (void) setValuetypeFor:(NSNotification*)note
{
    id object = [note object];
    NSMutableDictionary* info = (NSMutableDictionary*)[note userInfo];
    IDLValueType* value = [info objectForKey:@"valuetype"];
    IDLInterface* interface = [info objectForKey:@"interface"];
    id class = _className?NSClassFromString(_className):nil;
    if(value == self || object == nil || class == nil || class == [NSObject class])
        return;
    
    if(![object isKindOfClass:class])
        return;
    
    if(value == nil && interface == nil){
        [info setObject:self forKey:@"valuetype"];
    }else
    if(value && [self isDerivedFrom:value]){
        [info setObject:self forKey:@"valuetype"];
    }else
    if(!value && interface && [self isDerivedFromInterface:interface]){
        [info setObject:self forKey:@"valuetype"];
    }
}

+ (void) registerValuetype:(IDLValueType*)value forClass:(Class)objectClass
{
    if(!_cache)
        _cache = [[NSMutableDictionary allocWithZone:[self zone]]init];
    if(objectClass && value)
        [_cache setObject:value forKey:NSStringFromClass(objectClass)];
}

+ valuetypeForClass:(Class)class
{
    if(class)
        return [_cache objectForKey:NSStringFromClass(class)];
    else
        return nil;
}

+ (void) invalidateValuetypeCache
{
    [_cache removeAllObjects];
}

+ valuetypeForObject:object interface:(IDLInterface*)interface
{
    IDLValueType* value;
    
    if([object respondsToSelector:@selector(valuetype)])
        value = [object valuetype];
    else
        value = [self valuetypeForClass:[object class]];
    
    if([value isDerivedFromInterface:interface])
        return value;
    else{
        NSMutableDictionary* info = [NSMutableDictionary dictionaryWithObject:interface forKey:@"interface"];
        [[NSNotificationCenter defaultCenter]postNotificationName:IDLValuetypeNeededForObject
                object:object userInfo:info];
    
        value = [info objectForKey:@"valuetype"];
        if(value && object && 
					![object isKindOfClass:[NSDictionary class]] && 
					![object isKindOfClass:[IDLObject class]])
            [self registerValuetype:value forClass:[object class]];
        return value;
    }
}

+ valuetypeForObject:object
{
    IDLValueType* value = [self valuetypeForClass:[object class]];
    if(!value){
        NSMutableDictionary* info = [NSMutableDictionary dictionary];
        [[NSNotificationCenter defaultCenter]postNotificationName:IDLValuetypeNeededForObject
                object:object userInfo:info];
    
        value = [info objectForKey:@"valuetype"];
        if(value && object && 
				![object isKindOfClass:[NSDictionary class]] && ![object isKindOfClass:[IDLObject class]])
            [self registerValuetype:value forClass:[object class]];
    }
    return value;
}

- valuetypeForObject:object
{
    IDLValueType* value;
    
    if(!object || [self isValueBox]) // Can't be subclassed
        return self;
    
    if([object respondsToSelector:@selector(valuetype)])
        value = [object valuetype];
    else
        value = [IDLValueType valuetypeForClass:[object class]];
    
    if(![value isDerivedFrom:self]){ 
        NSMutableDictionary* info = [NSMutableDictionary dictionaryWithObject:self forKey:@"valuetype"];
        [[NSNotificationCenter defaultCenter]postNotificationName:IDLValuetypeNeededForObject
                object:object userInfo:info];
    
        value = [info objectForKey:@"valuetype"];
        if(value && object && 
				![object isKindOfClass:[NSDictionary class]] && ![object isKindOfClass:[IDLObject class]])
            [IDLValueType registerValuetype:value forClass:[object class]];
    }
    return value;
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	[super encodeWithCoder:coder];
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_name forKey:@"name"];
		[coder encodeObject:_support forKey:@"support"];
		[coder encodeObject:_className forKey:@"className"];
		[coder encodeInt:_modifier forKey:@"modifier"];
	}else{
		[coder encodeObject:_name];
		[coder encodeObject:_support];
		[coder encodeObject:_className];
		[coder encodeValueOfObjCType:@encode(short) at:&_modifier];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super initWithCoder:coder];
	if([coder allowsKeyedCoding]){
		_name = [[coder decodeObjectForKey:@"name"]retain];
		_support = [[coder decodeObjectForKey:@"support"]retain];
		_className = [[coder decodeObjectForKey:@"className"]retain];
		_modifier = [coder decodeIntForKey:@"modifier"];
	}else{
		_name = [[coder decodeObject]retain];
		_support = [[coder decodeObject]retain];
		_className = [[coder decodeObject]retain];
		[coder decodeValueOfObjCType:@encode(short) at:&_modifier];
	}
	return self;
}

@end
