
/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLGIOPVersion.h"
#import "CORBACoding.h"
#import "IDLPrimitive.h"
#import "IDLTCKind.h"

@implementation IDLGIOPVersion

static int _giop_max_minor = GIOP_MAX_MINOR;
static unsigned char _majorVersion = GIOP_MAJOR;
static unsigned char _minorVersion = GIOP_MINOR;

+ (void) setMaxMinorVersion:(unsigned)minor
{
    _giop_max_minor = minor;
}

+ (void) setDefaultIIOPVersion:(unsigned)major:(unsigned)minor
{
    _majorVersion = major;
    _minorVersion = minor;
}

+ (IDLGIOPVersion*)defaultIIOPVersion
{
    return [[[IDLGIOPVersion alloc]init]autorelease];
}

+ (void)initialize
{
    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
    if([IDLGIOPVersion class] == self){
        int maxVersion;
        NSString* string = [[NSUserDefaults standardUserDefaults]stringForKey:@"defaultIIOPVersion"];
        [[NSUserDefaults standardUserDefaults]registerDefaults:
            [NSDictionary dictionaryWithObjectsAndKeys:
                [NSNumber numberWithInt:_giop_max_minor],@"giop_max_minor", nil]];
        maxVersion = [[NSUserDefaults standardUserDefaults]integerForKey:@"giop_max_minor"];
        if(maxVersion >= 0)
            [self setMaxMinorVersion:maxVersion];
        
        if(string){
            NSArray* array = [string componentsSeparatedByString:@"."];
            _majorVersion = [[array objectAtIndex:0]intValue];
            if([array count] > 1)
                _minorVersion = [[array objectAtIndex:1]intValue];
        }
    }
    [pool release];
}

- initVersion:(unsigned)major:(unsigned)minor
{
    self = [super init];
    _major = major;
    _minor = minor;
    if(_minor > _giop_max_minor)
        _minor = _giop_max_minor;
    return self;
}

- init
{
    return [self initVersion:_majorVersion:_minorVersion];
}

- (void) setMajor:value
{
	[IDLPrimitive setValue:&_major fromObject:value ofKind:tk_octet];
//    const unsigned char* bytes = [value bytes];
//    if(bytes)
//        _major = bytes[0];
//    else
//        _major = 1;
}

- (void) setMinor:value
{
	[IDLPrimitive setValue:&_minor fromObject:value ofKind:tk_octet];
//    const unsigned char* bytes = [value bytes];
//    if(bytes)
//        _minor = bytes[0];
//    else
//        _minor = 0;
}

- major
{
	return [IDLPrimitive objectForValue:&_major ofKind:tk_octet];
//    return [NSData dataWithBytes:&_major length:1];
}

- minor
{
	return [IDLPrimitive objectForValue:&_minor ofKind:tk_octet];
//    return [NSData dataWithBytes:&_minor length:1];
}

- (unsigned) majorVersion
{
    return _major;
}

- (unsigned) minorVersion
{
    return _minor;
}

- (void) decodeVersion:coder
{
    [coder decodeChar:&_major];
    [coder decodeChar:&_minor];
}

- (void) encodeVersion:coder
{
    [coder encodeChar:_major];
    [coder encodeChar:_minor];
}

@end
