/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "VXConnectionObserver.h"
#import "VXConnection.h"

@implementation VXConnectionObserver

static id _observerClass = nil;
static NSHashTable *_observers = NULL;

+ (void) _acceptConnection:(NSNotification*)notification
{
    if(_observerClass){
        id connection = [notification object];
        // observer is NOT autoreleased
        id observer = [_observerClass observerWithConnection:connection];
        [observer connectionAccepted];
    }
}
+ (void)initialize
{
    if([VXConnectionObserver class] == self){
        _observers = NSCreateHashTable(NSNonRetainedObjectHashCallBacks,0);
        [[NSNotificationCenter defaultCenter]addObserver:self
                    selector:@selector(_acceptConnection:)
                    name:VXAcceptedConnection
                    object:nil];
    }
}

- (void)_invalidate:(NSNotification*)notification
{
    [self connectionDestroyed];
    [self release];
}

- initWithConnection:connection
{
    self = [super init];
    _connection = [connection retain];
    NSHashInsertKnownAbsent(_observers,self);
    [[NSNotificationCenter defaultCenter]addObserver:self
            selector:@selector(_invalidate:)
            name:VXInvalidateConnection
            object:_connection];
    return self;
}

- (void) dealloc
{
    NSHashRemove(_observers,self);
    [[NSNotificationCenter defaultCenter]removeObserver:self];
    [_connection release];
    [super dealloc];
}
+ lookupObserverWith:connection
{
    struct { @defs(VXConnectionObserver) } shadow;
    shadow.isa = [self class];
    shadow._connection = connection;
    return NSHashGet(_observers, &shadow);
}
+ observerWithConnection:connection
{
    id obj = [self lookupObserverWith:connection];
    if(!obj)
        obj = [[self alloc]initWithConnection:connection];
    return obj;
}
+ observerForCurrentRequest
{
    id connection = [VXConnection currentConnection];
    return [self lookupObserverWith:connection];
}
- (void) connectionAccepted
{
    
}
- (void) connectionDestroyed
{
    
}

+ observerClass
{
    return _observerClass;
}

+ (void) setObserverClass:newClass
{
    _observerClass = newClass;
}

- (BOOL)isEqual:(VXConnectionObserver*)other
{
    if(!other || other->isa != isa)
        return NO;
    return other && other->isa == isa && other->_connection == _connection;
}

- (unsigned)hash
{
    return [_connection hash];
}

- (NSString*)hostName
{
    return [_connection hostName];
}
- (NSNumber*)port
{
    return [_connection port];
}
- (NSString*)address
{
    return [_connection address];
}

@end
